#############################################################################
##
## DVD BACKUP - Shell script for backuping data on DVD media
## Copyright (C) 2005-2007, SILVERCODERS(tm)
## http://silvercoders.com
##
## Project homepage: http://silvercoders.com/index.php?page=DVD_Backup
##
## This program may be distributed and/or modified under the terms of the
## GNU General Public License version 2 as published by the Free Software
## Foundation and appearing in the file COPYING.GPL included in the
## packaging of this file.
##
## Please remember that any attempt to workaround the GNU General Public
## License using wrappers, pipes, client/server protocols, and so on
## is considered as license violation. If your program, published on license
## other than GNU General Public License version 2, calls some part of this
## code directly or indirectly, you have to buy commercial license.
## If you do not like our point of view, simply do not use the product.
##
## Licensees holding valid commercial license for this product
## may use this file in accordance with the license published by
## SILVERCODERS and appearing in the file COPYING.COM
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##
#############################################################################


function help()
{
	echo "SILVERCODERS DVD BACKUP version `cat VERSION`"
	echo "(C) 2005-2007 SILVERCODERS (http://silvercoders.com)"
	echo
	echo "dvd_backup.sh [--config <file_name>] --init|--add"
	echo
	echo "--config <file_name> - use specified config file (default is ./dvd_backup.conf)"
	echo "--init - initial backup (empty media)"
	echo "--add  - add next backup (media with backups)"
	echo
}

function create_temporary_dir()
{
	mkdir "$BACKUP_TEMP_DIR"
	if test $? -ne 0; then
		echo "Creating temporary directory failed!"
		exit 1
	fi
}

function remove_temporary_dir()
{
	rm -rf "$BACKUP_TEMP_DIR"
	if test $? -ne 0; then
		echo "Removing temporary directory failed!"
		exit 1
	fi
}

function backup_postgresql()
{
	if test -n "$BACKUP_POSTGRESQL_USER"; then
		echo Dumping PostgreSQL databases ...
		PGPASSWORD="$BACKUP_POSTGRESQL_PASSWORD" \
			pg_dumpall -U $BACKUP_POSTGRESQL_USER > "$BACKUP_TEMP_DIR/postgresql_backup.sql"
		if test $? -ne 0; then
			echo "Dumping PostgreSQL databases failed!"
			exit 1
		fi
		PGSQL_BACKUP_SIZE=`ls -lh "$BACKUP_TEMP_DIR/postgresql_backup.sql" | awk '{print $5}'`
		BACKUP_PATHS="$BACKUP_TEMP_DIR/postgresql_backup.sql $BACKUP_PATHS"
	fi
}

function backup_mysql()
{
	if test -n "$BACKUP_MYSQL_USER"; then
		echo Dumping MySQL databases ...
		mysqldump -u "$BACKUP_MYSQL_USER" --password="$BACKUP_MYSQL_PASSWORD" \
			--all-databases > "$BACKUP_TEMP_DIR/mysql_backup.sql"
		if test $? -ne 0; then
			echo "Dumping MySQL databases failed!"
			exit 1
		fi
		MYSQL_BACKUP_SIZE=`ls -lh "$BACKUP_TEMP_DIR/mysql_backup.sql" | awk '{print $5}'`
		BACKUP_PATHS="$BACKUP_TEMP_DIR/mysql_backup.sql $BACKUP_PATHS"
	fi
}

function backup_files()
{
	echo
	echo "Backup paths: $BACKUP_PATHS"
	echo "Exclude:      $BACKUP_EXCLUDE"
	echo
	for exclude in $BACKUP_EXCLUDE; do
		echo "$exclude" >> "$BACKUP_TEMP_DIR/dvdbackup_exclude.tmp"
	done
	EXCLUDE=`echo "$BACKUP_EXCLUDE" | sed "s/[^ ]\+/--exclude \"&\"/g"`
	tar -cjpvf "$BACKUP_TEMP_DIR/$FILE_NAME" $INCREMENTAL $BACKUP_PATHS --exclude-from="$BACKUP_TEMP_DIR/dvdbackup_exclude.tmp"
	res=$?
	if test $res -eq 1; then
		echo "WARNING: Some files were changed while being archived!"
	fi
	if test $res -ne 0 -a $res -ne 1; then
		echo "Compressing files failed!"
		exit 1
	fi
}

function calc_checksum()
{
	if test -z "$BACKUP_CALC_CHECKSUM"; then
		return
	elif test "$BACKUP_CALC_CHECKSUM" = sha1; then
		CHKSUM_CMD="sha1sum"
	elif test "$BACKUP_CALC_CHECKSUM" = md5; then
		CHKSUM_CMD="md5sum"
	else
		echo "Incorrect value of BACKUP_CALC_CHECKSUM!"
		exit 1
	fi
	echo "Calculating checksum ..."
	cd "$BACKUP_TEMP_DIR"
	$CHKSUM_CMD "$FILE_NAME" > "$FILE_NAME.$BACKUP_CALC_CHECKSUM"
	res=$?
	cd - > /dev/null
	if test $res -ne 0; then
		echo "Calculating checksum failed!"
		exit 1
	fi
}

function write_to_media()
{
	if test -n "$BACKUP_DEVICE_SPEED"; then
		SPEED="-speed=$BACKUP_DEVICE_SPEED"
	else
		SPEED=""
	fi
	if test -n "$BACKUP_CALC_CHECKSUM"; then
		CHKSUM="$BACKUP_TEMP_DIR/$FILE_NAME.$BACKUP_CALC_CHECKSUM"
	else
		CHKSUM=""
	fi
	growisofs $SPEED $REC_FLAG $BACKUP_DEVICE -R -J "$BACKUP_TEMP_DIR/$FILE_NAME" $CHKSUM
	if test $? -ne 0; then
		echo "Writting to media failed!"
		exit 1
	fi
}

function generate_report()
{
	DIRS=`echo "$BACKUP_PATHS" | sed "s/ /\n    /g"`
	EXCL=`echo "$BACKUP_EXCLUDE" | sed "s/ /\n    /g"`
	SIZE=`ls -lh "$BACKUP_TEMP_DIR/$FILE_NAME" | awk '{print $5}'`
	if test -n "$PGSQL_BACKUP_SIZE"; then
		PGSQL_REPORT="
 PostgreSQL backup file size (before compression): $PGSQL_BACKUP_SIZE"
	fi
	if test -n "$MYSQL_BACKUP_SIZE"; then
		MYSQL_REPORT="
 MySQL backup file size (before compression):      $MYSQL_BACKUP_SIZE"
	fi
	REPORT="
**********************************************************************
 SILVERCODERS DVD BACKUP Report - `date`
**********************************************************************

 DVD BACKUP successfuly recorded your important data to DVD media.
${PGSQL_REPORT}${MYSQL_REPORT}
 Backup file size (after compression):             $SIZE

 Archived files and directories:
    $DIRS

 Excluded file patterns:
    $EXCL

 You can sleep well now ;)

**********************************************************************
"
}

function send_email_notification()
{
	if test -n "$BACKUP_SMTP_HOST"; then
		echo Sending email notification ...
		RCPTS=""
		for rcpt in $BACKUP_MAIL_TO; do
			RCPTS=`echo "RCPT TO: $rcpt" && echo "$RCPTS"`
		done
		MAIL_TO=`echo "$BACKUP_MAIL_TO" | sed "s/ /, /g"`
		echo "HELO `hostname`" > "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		if test -n "$BACKUP_SMTP_USER"; then
			ENC_USER=`echo -n "$BACKUP_SMTP_USER" | uuencode -m - | head -n 2 | tail -n 1`
			ENC_PASS=`echo -n "$BACKUP_SMTP_PASSWORD" | uuencode -m - | head -n 2 | tail -n 1`
			echo "AUTH LOGIN" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
			echo "$ENC_USER" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
			echo "$ENC_PASS" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		fi
		echo "MAIL FROM: $BACKUP_MAIL_FROM" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "$RCPTS" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "DATA" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "To: $MAIL_TO" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "Subject: SILVERCODERS DVD BACKUP report" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "$REPORT" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "." >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		echo "QUIT" >> "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp"
		cat "$BACKUP_TEMP_DIR/dvdbackup_mail.tmp" | nc "$BACKUP_SMTP_HOST" 25
		if test $? -ne 0; then
			echo "Sending email notification failed!"
			exit 1
		fi
	fi
}

CONFIG_FILE="./dvd_backup.conf"
if test "$1" = "--config"; then
	if test $# -le 2; then
		help
		exit 1
	fi
	CONFIG_FILE="$2"
	shift
	shift
fi

case "$1" in
	"--init" )
		echo "Performing initial backup ..."
		REC_FLAG=-Z
		INCREMENTAL=
		;;
	"--add" )
		echo "Adding new backup (incremental)..."
		REC_FLAG=-M
		if test -f ./.backup_last_date; then
			INCREMENTAL="--newer=`cat ./.backup_last_date`"
		else
			echo ".backup_last_date file not found in current directory!"
			echo "Performing full backup!"
			INCREMENTAL=
		fi
		;;
	* )
		help
		exit 1
		;;
esac

if ! test -f "$CONFIG_FILE"; then
	echo "Configuration file $CONFIG_FILE not found!"
	echo
	exit 1
fi
. "$CONFIG_FILE"

CURR_DATE=`date +%Y-%m-%d`
FILE_NAME=backup-`hostname`-`date +%Y%m%d-%H%M%S`.tar.bz2
echo

create_temporary_dir
backup_postgresql
backup_mysql
backup_files
calc_checksum
write_to_media
generate_report
send_email_notification
remove_temporary_dir

echo "$CURR_DATE" > ./.backup_last_date
echo "$REPORT"
